!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA GSFC Land Data Toolkit (LDT) V1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
!BOP
! 
! !ROUTINE: readSMOPSsmObs
! \label{readSMOPSsmObs}
! 
! !REVISION HISTORY: 
!  8 May 2013: Sujay Kumar, Initial Specification
! 
! !INTERFACE: 
subroutine readSMOPSsmObs(n)
! !USES:   
  use ESMF
  use LDT_coreMod,      only : LDT_rc
  use LDT_timeMgrMod,   only : LDT_get_julss
  use LDT_logMod,       only : LDT_logunit, LDT_getNextUnitNumber, &
       LDT_releaseUnitNumber
  use LDT_DAobsDataMod
  use SMOPSsm_obsMod, only : SMOPSsmobs
  use map_utils

  implicit none
! !ARGUMENTS: 
  integer, intent(in) :: n
! 
! !DESCRIPTION: 
! 
! This subroutine provides the data reader for the SMOPS
! soil moisture retrieval product. The data has many layers 
! and the reader provides the options to select the 
! desired layer(s).
! 

!
!EOP

  real              :: timenow
  logical           :: alarmCheck
  logical           :: file_exists
  integer           :: c,r,i,j
  character*100     :: fname
  real              :: smobs(LDT_rc%lnc(n)*LDT_rc%lnr(n))

!-----------------------------------------------------------------------
! It is assumed that CDF is computed using daily observations. 
!-----------------------------------------------------------------------
  SMOPSsmobs(n)%smobs = LDT_rc%udef
  smobs = LDT_rc%udef

  call create_SMOPSsm_filename(SMOPSsmobs(n)%odir, &
       LDT_rc%yr, LDT_rc%mo, LDT_rc%da, fname)
  
  inquire(file=trim(fname),exist=file_exists)
  if(file_exists) then
     
     write(LDT_logunit,*) '[INFO] Reading ..',trim(fname)
     call read_SMOPS_data(n, fname,smobs)
     write(LDT_logunit,*) '[INFO] Finished reading ',trim(fname)

     do r=1,LDT_rc%lnr(n)
        do c=1,LDT_rc%lnc(n)
           if(smobs(c+(r-1)*LDT_rc%lnc(n)).ne.-9999.0) then 
              SMOPSsmobs(n)%smobs(c,r) = smobs(c+(r-1)*LDT_rc%lnc(n))
           endif
        enddo
     enddo
  else
     SMOPSsmobs(n)%smobs = LDT_rc%udef
  endif

  call LDT_logSingleDAobs(n,LDT_DAobsData(n)%soilmoist_obs,&
       SMOPSsmobs(n)%smobs,vlevel=1)

end subroutine readSMOPSsmObs


!BOP
! 
! !ROUTINE: read_SMOPS_data
! \label(read_SMOPS_data)
!
! !INTERFACE:
subroutine read_SMOPS_data(n, fname, smobs_ip)
! 
! !USES:   
  use grib_api
  use LDT_coreMod,  only : LDT_rc, LDT_domain
  use LDT_logMod
  use map_utils,    only : latlon_to_ij
  use SMOPSsm_obsMod, only : SMOPSsmobs

  implicit none
!
! !INPUT PARAMETERS: 
! 
  integer                       :: n 
  character (len=*)             :: fname
  real                          :: smobs_ip(LDT_rc%lnc(n)*LDT_rc%lnr(n))


! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
!  This subroutine reads the SMOPS grib2 file and applies the data
!  quality flags to filter the data. The retrievals are rejected when 
!  the estimated error is above a predefined threshold (the recommeded
!  value is 5%). 
! 
! The information from gribtab is: 
!{ 2, 0, 7, 1, 3, 210, "BLENDEDSM", "NOAA Blended Liquid Volumetric Soil Moisture (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 211, "AMSRESM", "NOAA AMSR-E Liquid Volumetric Soil Moisture (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 212, "SMOSSM", "SMOS Liquid Volumetric Soil Moisture (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 213, "ASCATSM", "ASCAT Liquid Volumetric Soil Moisture (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 214, "WINDSATSM", "NOAA WindSat Liquid Volumetric Soil Moisture (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 215, "RESSM1", "Reserved Liquid Volumetric Soil Moisture 1 (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 216, "RESSM2", "Reserved Liquid Volumetric Soil Moisture 2 (Non-Frozen)", "m3m-3"},
!{ 2, 0, 7, 1, 3, 217, "BLENDEDSMHR", "Observation Hour for BLENDEDSM", "-"},
!{ 2, 0, 7, 1, 3, 218, "BLENDEDSMMN", "Observation Minute for BLENDEDSM", "-"},
!{ 2, 0, 7, 1, 3, 219, "AMSRESMHR", "Observation Hour for AMSRESM", "-"},
!{ 2, 0, 7, 1, 3, 220, "AMSRESMMN", "Observation Minute for AMSRESM", "-"},
!{ 2, 0, 7, 1, 3, 221, "SMOSSMHR", "Observation Hour for SMOSSM", "-"},
!{ 2, 0, 7, 1, 3, 222, "SMOSSMMN", "Observation Minute for SMOSSM", "-"},
!{ 2, 0, 7, 1, 3, 223, "ASCATSMHR", "Observation Hour for ASCATSM", "-"},
!{ 2, 0, 7, 1, 3, 224, "ASCATSMMN", "Observation Minute for ASCATSM", "-"},
!{ 2, 0, 7, 1, 3, 225, "WINDSATSMHR", "Observation Hour for WINDSATSM", "-"},
!{ 2, 0, 7, 1, 3, 226, "WINDSATSMMN", "Observation Minute for WINDSATSM", "-"},
!{ 2, 0, 7, 1, 3, 227, "RESSM1HR", "Observation Hour for RESSM1", "-"},
!{ 2, 0, 7, 1, 3, 228, "RESSM1MN", "Observation Minute for RESSM1", "-"},
!{ 2, 0, 7, 1, 3, 229, "RESSM2HR", "Observation Hour for RESSM2", "-"},
!{ 2, 0, 7, 1, 3, 230, "RESSM2MN", "Observation Minute for RESSM2", "-"},
!{ 2, 0, 7, 1, 3, 231, "BLENDEDSMQA", "NOAA Blended Liquid Volumetric Soil Moisture QA", "-"},
!{ 2, 0, 7, 1, 3, 232, "AMSRESMQA", "NOAA AMSR-E Liquid Volumetric Soil Moisture QA", "-"},
!{ 2, 0, 7, 1, 3, 233, "SMOSSMQA", "SMOS Liquid Volumetric Soil Moisture QA", "-"},
!{ 2, 0, 7, 1, 3, 234, "ASCATSMQA", "ASCAT Liquid Volumetric Soil Moisture QA", "-"},
!{ 2, 0, 7, 1, 3, 235, "WINDSATSMQA", "NOAA WindSat Liquid Volumetric Soil Moisture QA", "-"},
!{ 2, 0, 7, 1, 3, 236, "RESSM1QA", "Reserved Liquid Volumetric Soil Moisture 1 QA", "-"},
!{ 2, 0, 7, 1, 3, 237, "RESSM2QA", "Reserved Liquid Volumetric Soil Moisture 2 QA", "-"},

!  The arguments are: 
!  \begin{description}
!  \item[n]            index of the nest
!  \item[fname]        name of the SMOPS AMSR-E file
!  \item[smobs\_ip]    soil moisture data processed to the LDT domain
! \end{description}
!
!
!EOP
  INTEGER*2, PARAMETER :: FF = 255
  real,    parameter  :: err_threshold = 5 ! in percent
  integer, parameter  :: param_ASCAT= 213, param_ASCAT_qa = 234
  integer, parameter  :: param_windsat = 214, param_windsat_qa = 235
  integer, parameter  :: param_SMOS = 212, param_SMOS_qa = 233
  real                :: sm_ascat(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  real                :: sm_ascat_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_ascat_qa(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  integer*2           :: sm_ascat_qa_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_windsat(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  real                :: sm_windsat_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_windsat_qa(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  integer*2           :: sm_windsat_qa_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_smos(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  real                :: sm_smos_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_smos_qa(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  integer*2           :: sm_smos_qa_t(SMOPSsmobs(n)%smopsnc*&
  SMOPSsmobs(n)%smopsnr)
  real                :: sm_data(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  logical*1           :: sm_data_b(SMOPSsmobs(n)%smopsnc*&
       SMOPSsmobs(n)%smopsnr)
  logical*1           :: smobs_b_ip(LDT_rc%lnc(n)*LDT_rc%lnr(n))

  integer             :: c,r,i,j,kk
  integer             :: ftn,iret,igrib,nvars
  integer             :: param_num
  logical             :: var_found_ascat
  logical             :: var_found_windsat
  logical             :: var_found_smos
  real                :: err, ql

  call grib_open_file(ftn,trim(fname), 'r',iret)
  if(iret.ne.0) then 
     write(LDT_logunit,*) 'Could not open file: ',trim(fname)
     call LDT_endrun()
  endif
  call grib_multi_support_on

  do
     call grib_new_from_file(ftn,igrib,iret)

     if ( iret == GRIB_END_OF_FILE ) then
        exit
     endif

     call grib_get(igrib, 'parameterNumber',param_num, iret)
     call LDT_verify(iret, &
          'grib_get: parameterNumber failed in readSMOPSsmobs')

     var_found_ascat = .false. 
     if(SMOPSsmobs(n)%useASCAT.eq.1) then 
        if(param_num.eq.param_ascat) then 
           var_found_ascat = .true.
        endif
     endif
     
     if(var_found_ascat) then
        call grib_get(igrib, 'values',sm_ascat,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_ascat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   sm_ascat(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc)
           enddo
        enddo     
     endif

     var_found_ascat = .false. 
     if(SMOPSsmobs(n)%useASCAT.eq.1) then 
        if(param_num.eq.param_ascat_qa) then 
           var_found_ascat = .true.
        endif
     endif
     
     if(var_found_ascat) then
        call grib_get(igrib, 'values',sm_ascat_qa,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_ascat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   INT(sm_ascat_qa(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc))
           enddo
        enddo       
     endif

!windsat
     var_found_windsat = .false. 
     if(SMOPSsmobs(n)%useWINDSAT.eq.1) then 
        if(param_num.eq.param_windsat) then 
           var_found_windsat = .true.
        endif
     endif
     
     if(var_found_windsat) then
        call grib_get(igrib, 'values',sm_windsat,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_windsat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   sm_windsat(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc)
           enddo
        enddo     
        
     endif

     var_found_windsat = .false. 
     if(SMOPSsmobs(n)%useWINDSAT.eq.1) then 
        if(param_num.eq.param_windsat_qa) then 
           var_found_windsat = .true.
        endif
     endif
     
     if(var_found_windsat) then
        call grib_get(igrib, 'values',sm_windsat_qa,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_windsat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   INT(sm_windsat_qa(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc))
           enddo
        enddo       
     endif
!smos
     var_found_smos = .false. 
     if(SMOPSsmobs(n)%useSMOS.eq.1) then 
        if(param_num.eq.param_smos) then 
           var_found_smos = .true.
        endif
     endif
     
     if(var_found_smos) then
        call grib_get(igrib, 'values',sm_smos,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_smos_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   sm_smos(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc)
           enddo
        enddo     
        
     endif

     var_found_smos = .false. 
     if(SMOPSsmobs(n)%useSMOS.eq.1) then 
        if(param_num.eq.param_smos_qa) then 
           var_found_smos = .true.
        endif
     endif
     
     if(var_found_smos) then
        call grib_get(igrib, 'values',sm_smos_qa,iret)
        call LDT_warning(iret,'error in grib_get:values in readSMOPSsmObs')
        
        do r=1,SMOPSsmobs(n)%smopsnr
           do c=1,SMOPSsmobs(n)%smopsnc
              sm_smos_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = &
                   INT(sm_smos_qa(c+((SMOPSsmobs(n)%smopsnr-r+1)-1)*&
                   SMOPSsmobs(n)%smopsnc))
           enddo
        enddo       
     endif
     
     call grib_release(igrib,iret)
     call LDT_verify(iret, 'error in grib_release in readSMOPSsmObs')
  enddo

  call grib_close_file(ftn)

  if(SMOPSsmobs(n)%useASCAT.eq.1) then 
     do r=1, SMOPSsmobs(n)%smopsnr
        do c=1, SMOPSsmobs(n)%smopsnc
           if(sm_ascat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc).ne.9999) then 
              !estimated error
              err = ISHFT(sm_ascat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),-8)
              !quality flag - not used currently
              ql = IAND(sm_ascat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),FF)
              
              if(err.lt.err_threshold) then 
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .true. 
              else
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false.
                 sm_ascat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              endif
           else
              sm_ascat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false. 
           endif
        enddo
     enddo
!--------------------------------------------------------------------------
! Interpolate to the LDT running domain
!-------------------------------------------------------------------------- 
     call neighbor_interp(LDT_rc%gridDesc(n,:),&
          sm_data_b, sm_ascat_t, smobs_b_ip, smobs_ip, &
          SMOPSsmobs(n)%smopsnc*SMOPSsmobs(n)%smopsnr, &
          LDT_rc%lnc(n)*LDT_rc%lnr(n), &
          LDT_domain(n)%lat, LDT_domain(n)%lon,&
          SMOPSsmobs(n)%n11, LDT_rc%udef, iret)

  endif

  if(SMOPSsmobs(n)%useWINDSAT.eq.1) then 
     do r=1, SMOPSsmobs(n)%smopsnr
        do c=1, SMOPSsmobs(n)%smopsnc
           if(sm_windsat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc).ne.9999) then 
              !estimated error
              err = ISHFT(sm_windsat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),-8)
              !quality flag - not used currently
              ql = IAND(sm_windsat_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),FF)
              
              if(err.lt.err_threshold) then 
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .true. 
              else
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false.
                 sm_windsat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              endif
           else
              sm_windsat_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false. 
           endif
        enddo
     enddo
!--------------------------------------------------------------------------
! Interpolate to the LDT running domain
!-------------------------------------------------------------------------- 
     call neighbor_interp(LDT_rc%gridDesc(n,:),&
          sm_data_b, sm_windsat_t, smobs_b_ip, smobs_ip, &
          SMOPSsmobs(n)%smopsnc*SMOPSsmobs(n)%smopsnr, &
          LDT_rc%lnc(n)*LDT_rc%lnr(n), &
          LDT_domain(n)%lat, LDT_domain(n)%lon,&
          SMOPSsmobs(n)%n11, LDT_rc%udef, iret)
  endif

  if(SMOPSsmobs(n)%useSMOS.eq.1) then 
     do r=1, SMOPSsmobs(n)%smopsnr
        do c=1, SMOPSsmobs(n)%smopsnc
           if(sm_smos_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc).ne.9999) then 
              !estimated error
              err = ISHFT(sm_smos_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),-8)
              !quality flag - not used currently
              ql = IAND(sm_smos_qa_t(c+(r-1)*SMOPSsmobs(n)%smopsnc),FF)
              
              if(err.lt.err_threshold) then 
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .true. 
              else
                 sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false.
                 sm_smos_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              endif
           else
              sm_smos_t(c+(r-1)*SMOPSsmobs(n)%smopsnc) = LDT_rc%udef
              sm_data_b(c+(r-1)*SMOPSsmobs(n)%smopsnc) = .false. 
           endif
        enddo
     enddo
!--------------------------------------------------------------------------
! Interpolate to the LDT running domain
!-------------------------------------------------------------------------- 
     call neighbor_interp(LDT_rc%gridDesc(n,:),&
          sm_data_b, sm_smos_t, smobs_b_ip, smobs_ip, &
          SMOPSsmobs(n)%smopsnc*SMOPSsmobs(n)%smopsnr, &
          LDT_rc%lnc(n)*LDT_rc%lnr(n), &
          LDT_domain(n)%lat, LDT_domain(n)%lon,&
          SMOPSsmobs(n)%n11, LDT_rc%udef, iret)
  endif

end subroutine read_SMOPS_data

!BOP
! !ROUTINE: create_SMOPSsm_filename
! \label{create_SMOPSsm_filename}
! 
! !INTERFACE: 
subroutine create_SMOPSsm_filename(ndir, yr, mo,da, filename)
! !USES:   

  implicit none
! !ARGUMENTS: 
  character(len=*)  :: filename
  integer           :: yr, mo, da
  character (len=*) :: ndir
! 
! !DESCRIPTION: 
!  This subroutine creates the SMOPS filename based on the time and date 
! 
!  The arguments are: 
!  \begin{description}
!  \item[ndir] name of the SMOPS soil moisture directory
!  \item[yr]  current year
!  \item[mo]  current month
!  \item[da]  current day
!  \item[filename] Generated RT SMOPS filename
! \end{description}
!EOP

  character (len=4) :: fyr
  character (len=2) :: fmo,fda
  
  write(unit=fyr, fmt='(i4.4)') yr
  write(unit=fmo, fmt='(i2.2)') mo
  write(unit=fda, fmt='(i2.2)') da
 
  filename = trim(ndir)//'/'//trim(fyr)//'/NPR_SMOPS_CMAP_D' &
       //trim(fyr)//trim(fmo)//trim(fda)//'.gr2'
  
end subroutine create_SMOPSsm_filename
